/* This file is part of turtle, a turtle-graphics system for Scheme.

Copyright (C) 2008  Luca Saiu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */


#ifndef TURTLE_H_
#define TURTLE_H_

#include <stdbool.h>
#include "graphics.h"
#include "compiletimeparameters.h"
#include "debug.h"

/* A turtle is just represented as a record with the
   relevant fields: */
struct turtle_turtle{
  /* Position: */
  float x, y;
  
  /* Orientation, in radiants: */
  float orientation;
  
  /* Cursor height, in pixels: */
  float cursor_height;
  
  /* Color; note that this is *not* a pointer, it's a struct,
     because it's meant to be modifiable in-place: */
  struct turtle_color color;
  
  /* Is the pen down? */
  bool pen_down;

  /* Is the turtle hidden? */
  bool hidden;
}; // struct

/* We normally refer a turtle via a pointer: */
typedef struct turtle_turtle* turtle_turtle_t;

/* Initialize an already-allocated turtle: */
void turtle_initialize_turtle(turtle_turtle_t turtle);

/* Allocate and initialize a new turtle: */
turtle_turtle_t turtle_make_turtle(void);

/* Destroy and deallocate an existing turtle: */
void turtle_destroy_turtle(turtle_turtle_t turtle);

/* Draw the cursor for the given turte (as a cursor, of course; see graphics.h): */
void turtle_draw_cursor(turtle_turtle_t turtle);

/* Turtle primitives: */
void turtle_set_pen(turtle_turtle_t turtle, const bool down);
void turtle_set_color(turtle_turtle_t turtle, const turtle_color_t color);
void turtle_set_cursor_height(turtle_turtle_t turtle, const float height);
void turtle_forward(turtle_turtle_t turtle, const float distance);
void turtle_backward(turtle_turtle_t turtle, const float distance);
void turtle_left_radiants(turtle_turtle_t turtle, const float angle);
void turtle_right_radiants(turtle_turtle_t turtle, const float angle);
void turtle_left_degrees(turtle_turtle_t turtle, const float angle);
void turtle_right_degrees(turtle_turtle_t turtle, const float angle);
void turtle_set_orientation_radiants(turtle_turtle_t turtle, const float new_orientation);
void turtle_set_orientation_degrees(turtle_turtle_t turtle, const float new_orientation);
void turtle_set_position(turtle_turtle_t turtle, const float new_x, const float new_y);
void turtle_goto(turtle_turtle_t turtle, const float new_x, const float new_y);
void turtle_home(turtle_turtle_t turtle);
void turtle_hide(turtle_turtle_t turtle);
void turtle_show(turtle_turtle_t turtle);

#endif // #ifndef TURTLE_H_
